
/*******************************************************************
				CGUI 3
			Client per server GFG 1.0

Realizzato in Java nel Gennaio 1999 per il corso di Impianti di Elaborazione
	presso l'universit di RomaTre

Consigliata versione di Java 1.2, necessaria almeno 1.1
********************************************************************/

import java.awt.event.*;
import java.util.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import BufferCircolare; //classe che gestisce la cache

/********************************************************************
CGUI3  l'interfaccia grafica del client. Oltre a visualizzare i 
componenti grafici contiene funzioni ausiliarie per gestire la cache,
per salvare dati su file, etc. etc.
Sono definite al suo interno varie "classi interne": ci rende il 
corpo di questa classe piuttosto grande, ma consente di limitare il
numero di classi che non avrebbero senso slegate da questa interfaccia
grafica.
*********************************************************************/

public class CGUI3 extends Frame {
  //seguono valori di default dell'interfaccia
  static final int DROWS=18; //numero di righe del testo per visualizzare i dati
  static final   int DCOLS=60;//numero colonne
  static final int DX=500;//larghezza della finestra
  static final int DY=600;//altezza della finestra
  static final int DTCP_RECEIVE_BUFFER= 8000;//in byte, buffer ricezione per TCP
  static final int DUDP_RECEIVE_BUFFER= 1500;//analogo per UDP
  static final String DCACHE_DIRECTORY="cache";//directory di salvataggio della cache
  static final String DCFG_FILE="gfg.cfg";//nome del file di configurazione 
  
  static final String DTEMPFILE="tempfile.tmp";//file di swap
  static final String DSPECIAL_SYMBOL="$$";//questo simbolo sostituisce nella riga di comando il nome della risorsa caricata
  
  //ora alle variabili vengono assegnati i valori di default o delle costanti
  int ROWS=DROWS;
  int COLS=DCOLS;
  int X=DX;
  int Y=DY;
  int TCP_RECEIVE_BUFFER= DTCP_RECEIVE_BUFFER;
  int UDP_RECEIVE_BUFFER= DUDP_RECEIVE_BUFFER;
  static final int BUT_SIZE= 50;
  static final int UP_BORDER= 50;//bordo dal lato superiore della finestra
  static final int DOWN_BORDER= 60;//dal lato inferiore
  static final int LEFT_BORDER= 30;
  static final int GAP=35;//spazio tra i componenti
  static final int HTTP_STANDARD_PORT =8080;//porta di default del SERVER	
  String CACHE_DIRECTORY=DCACHE_DIRECTORY;
  static final String WORKING_DIRECTORY="."+File.separator;
  String CFG_FILE=DCFG_FILE;
  static final int BYTES_PER_ROW=30;//per i salvataggi su file in modo strutturato
  static final int CACHE_SIZE=100;//ampiezza della cache
  static final String URL_EXTENSION=".url";
  String TEMPFILE=DTEMPFILE;
  String SPECIAL_SYMBOL=DSPECIAL_SYMBOL;
  static final int DEFAULT_TIMEOUT=3000;//timeout di 3 secondi
  byte[] buf=new byte[BYTES_PER_ROW];
   
  BufferCircolare bc; 
  http_handler hh;   //gestisce la lettura da socket 
  int timeout;    
  URL u;
  static boolean verbose=false;
  //definizione dei componenti grafici
  Button backbut=new Button("Back"),
    fwdbut = new Button("Forward"),
    reloadbut=new Button("Reload"), //attenzione il nome  usato altrove
    sendbut=new Button("Send"),
    closebut=new Button("Close"),
    
    listbut=new Button("List");
  
  CheckboxGroup cbg = new CheckboxGroup();
  Checkbox tcp = new Checkbox("TCP", cbg, true);
  Checkbox udp = new Checkbox("UDP", cbg, false);
  
  Label urllab=new Label("Location");
  
  TextArea actiontext=new TextArea(ROWS,COLS);
  
  TextField urltext=new TextField(COLS-10);
  
  
  //**************MENU 
  
  MenuBar mb1 = new MenuBar();
  Menu f = new Menu("File");
  
  MenuItem[] file = {
    
    new MenuItem("Open"),
    new MenuItem("Save"),
    new MenuItem("Clear Screen"),
    new MenuItem("Exit", 
		 new MenuShortcut(KeyEvent.VK_E))
  };
  Menu TimeOut=new Menu("Set Timeout");
  MenuItem[] time = {
    
    new MenuItem("1 second"),
    new MenuItem("2 seconds"),
    new MenuItem("5 seconds"),
    
    new MenuItem("10 seconds"),
    new MenuItem("infinite")
    
  };
  Menu o=new Menu("Options");
  MenuItem[] options={
    new MenuItem("Clear cache"),
    TimeOut  ,new MenuItem("Advanced")
    
  };
  
  Menu help=new Menu("Help");
  MenuItem[] helpitems={new MenuItem("Help..")};  
  // Inizializzazione
    {
      ML ml=new ML();//si occupa degli eventi del menu file
      CCL ccl=new CCL();//si occupa degli eventi sul menu options e help
      //vengono assegnati i gestori degli eventi
      file[0].setActionCommand("Open");
      file[0].addActionListener(ml);
      file[1].setActionCommand("Save");
      file[1].addActionListener(ml);
      
      file[2].setActionCommand("Clear");
      file[2].addActionListener(ml);
      file[3].setActionCommand("Exit");
      file[3].addActionListener(ml);
      
      options[0].setActionCommand("ClearCache");
      options[0].addActionListener(ccl);
      options[1].setActionCommand("TimeOut");
      options[1].addActionListener(ccl);
      options[2].setActionCommand("Advanced");
      options[2].addActionListener(ccl);
      helpitems[0].setActionCommand("help");
      helpitems[0].addActionListener(ccl);
      
      time[0].setActionCommand("TTT1");
      time[0].addActionListener(ccl);
      time[1].setActionCommand("TTT2");
      time[1].addActionListener(ccl);
      time[2].setActionCommand("TTT5");
      time[2].addActionListener(ccl);
      time[3].setActionCommand("TTT10");
      time[3].addActionListener(ccl);
      time[4].setActionCommand("TTTINF");
      time[4].addActionListener(ccl);
    }
  
  
  /** questa classe gestisce la sottofinestra che si apre alla scelta 
      delle opzioni avanzate nel menu. La visualizza, ne setta i valori, 
      gestisce i tre bottoni che consentono di tornare indietro, confermare 
      le scelte fatte, resettare tutti i valori a quelli di default.**/
  class Configuration 
{
  
  class okL implements ActionListener //gestore del bottone ok
  {
    public void actionPerformed(ActionEvent e)
      {
	
	
	ROWS=(new Integer(fields[0].getText())).intValue();
	COLS=(new Integer(fields[1].getText())).intValue();
	X=(new Integer(fields[2].getText())).intValue();
	Y=(new Integer(fields[3].getText())).intValue();
	TCP_RECEIVE_BUFFER=(new Integer(fields[4].getText())).intValue();
	UDP_RECEIVE_BUFFER=(new Integer(fields[5].getText())).intValue();
	
	
	String CACHE_DIRECTORY=fields[6].getText();
	String CFG_FILE=fields[7].getText();
	String TEMPFILE=fields[8].getText();
	String SPECIAL_SYMBOL=fields[9].getText();
	

	cancelbut.removeActionListener(okl);
	okbut.removeActionListener(cancell);
	defbut.removeActionListener(defl);
	d.dispose();
      }
    
  }
  
  class defL implements ActionListener //bottone default
  {
    public void actionPerformed(ActionEvent e)
      {
	
	ROWS=DROWS;
	COLS=DCOLS;
	X=DX;
	Y=DY;
	TCP_RECEIVE_BUFFER= DTCP_RECEIVE_BUFFER;
	UDP_RECEIVE_BUFFER= DUDP_RECEIVE_BUFFER;
	CACHE_DIRECTORY=DCACHE_DIRECTORY;
	CFG_FILE=DCFG_FILE;
	TEMPFILE=DTEMPFILE;
	SPECIAL_SYMBOL=DSPECIAL_SYMBOL;
	fields[0].setText((new Integer(ROWS)).toString());fields[1].setText((new Integer(COLS)).toString());
	fields[2].setText((new Integer(X)).toString());fields[3].setText((new Integer(Y)).toString());
	fields[4].setText(new Integer(TCP_RECEIVE_BUFFER).toString());fields[5].setText(new Integer(UDP_RECEIVE_BUFFER).toString());
	
	fields[6].setText(CACHE_DIRECTORY);fields[7].setText(CFG_FILE);
	fields[8].setText(TEMPFILE);fields[9].setText(SPECIAL_SYMBOL);
      }
    
  }
  
  
  class cancelL implements ActionListener//bottone cancel
  {
    public void actionPerformed(ActionEvent e)
      {
	cancelbut.removeActionListener(cancell);
	okbut.removeActionListener(okl);
	defbut.removeActionListener(defl);
	d.dispose();
      }
    
  }
  

  Frame d=new Frame(/*CGUI3.this,*/"Configurazione"/*,true*/);
  
  Button	  okbut=new Button("Okay"),
    defbut=new Button("Defaults"),
    cancelbut=new Button("Cancel");	
  Label labels[]={new Label(fill("ROWS")),new Label(fill("COLUMNS")),
		  new Label(fill("X size")),new Label(fill("Y size")),
		  new Label(fill("Buffer ricezione TCP")),new Label(fill("buffer ricezione UDP")),
		  new Label(fill("Cache directory")),new Label(fill("Config file")),
		  new Label(fill("Temp File")),new Label(fill("Special symbol"))};
  
  TextField fields[]={new TextField(15),new TextField(15),
		      new TextField(15),new TextField(15),new TextField(15),new TextField(15),
		      new TextField(15),new TextField(15),
		      new TextField(15),new TextField(15)};
  okL okl; cancelL cancell; defL defl;
  
  
  Configuration()
      { d.setSize(200,600);
      
      fields[0].setText((new Integer(ROWS)).toString());fields[1].setText((new Integer(COLS)).toString());
      fields[2].setText((new Integer(X)).toString());fields[3].setText((new Integer(Y)).toString());
      fields[4].setText(new Integer(TCP_RECEIVE_BUFFER).toString());fields[5].setText(new Integer(UDP_RECEIVE_BUFFER).toString());

      fields[6].setText(CACHE_DIRECTORY);fields[7].setText(CFG_FILE);
      fields[8].setText(TEMPFILE);fields[9].setText(SPECIAL_SYMBOL);
      okl=new okL();
      cancell=new cancelL();
      defl=new defL();
      okbut.addActionListener(okl);
      cancelbut.addActionListener(cancell);
      defbut.addActionListener(defl);
      d.setLayout(new FlowLayout());
      
      for (int i=0;i<labels.length;i++) {d.add(labels[i]);
      d.add(fields[i]);}
      d.add(okbut);
      d.add(defbut);
      d.add(cancelbut);
      d.setVisible(true);
      d.requestFocus();
      
      d.addWindowListener(
			  new WindowAdapter() {
	public void windowClosing(WindowEvent e){
	  if (verbose) System.out.println("Window Closing");
	  //  annulla.removeActionListener(annullal);
	  
	  d.dispose();
	}
      });
      }
  
  
  }
  
  /** ridefiniamo il metodo paint ereditato da Frame, cos da poter
      posizionare i componenti a piacere. I valori danno un risultato decente su
      1024x768 ... */
  public void paint(Graphics h) 
    {
      setSize(X,Y);
      
      backbut.setLocation(LEFT_BORDER,UP_BORDER);
      backbut.setSize(BUT_SIZE,BUT_SIZE);
      fwdbut.setLocation(LEFT_BORDER+BUT_SIZE+GAP,UP_BORDER);
      fwdbut.setSize(BUT_SIZE,BUT_SIZE);
      reloadbut.setLocation(2*BUT_SIZE+2*GAP+LEFT_BORDER,UP_BORDER);
      reloadbut.setSize(BUT_SIZE,BUT_SIZE);
      listbut.setLocation(LEFT_BORDER+3*BUT_SIZE+3*GAP,UP_BORDER);
      listbut.setSize(BUT_SIZE,BUT_SIZE);
      
      actiontext.setLocation(LEFT_BORDER,Y/3);
      
      if (verbose) System.out.println("ROWS="+ROWS+" COLS="+COLS);
      actiontext.setRows(ROWS);
      actiontext.setColumns(COLS);
      
      urltext.setLocation(LEFT_BORDER+2*GAP,Y/3-GAP); 
      urltext.setColumns(COLS-10);
      
      closebut.setLocation(X/3*2, Y-DOWN_BORDER);
      closebut.setSize(BUT_SIZE,BUT_SIZE);
      sendbut.setSize(BUT_SIZE,BUT_SIZE);
      sendbut.setLocation(X/3,Y- DOWN_BORDER);
      urllab.setLocation(LEFT_BORDER,Y/3-GAP);
      tcp.setLocation(2*X/3+GAP,UP_BORDER+GAP);
      udp.setLocation(2*X/3+60+GAP,UP_BORDER+GAP);
      
    }
  
  /** gestore eventi del menu options**/
  class CCL implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      MenuItem target = (MenuItem)e.getSource();
      String actionCommand = 
	target.getActionCommand();
      if(actionCommand.startsWith("TTT")) {// un comando sui timeout
	String temp=actionCommand.substring(3);
	if (verbose) System.out.println("TTT s="+temp);
	if (temp.startsWith("INF")) timeout=0;//richiede timeout infinito
	else
	  {
	    Integer i=new Integer(temp);
	    if (verbose) System.out.println("TTT i="+i);
	    timeout=i.intValue()*1000;//nella stringa di comando c' il numero di secondi, la funzione prende i millisecondi
	  }
      }
    if(actionCommand.startsWith("help")) {// un comando sull'aiuto
      
      if (verbose) System.out.println("HELP!");

Help h= new Help();
      
    }
    
    if(actionCommand.startsWith("Advanced")) {
      
      if (verbose) System.out.println("Config");//apre la finestra di configurazione
      Configuration cfg=new Configuration();
    }
    
    if(actionCommand.equals("ClearCache")) {//ripulisce completamente la cache
      File ff;
      if (verbose) System.out.println("Clearing!!");
      for (int i=0;i<CACHE_SIZE;i++){
	ff=new File(WORKING_DIRECTORY+CACHE_DIRECTORY,(new Integer(i)).toString());
	ff.delete();
	ff=new File(WORKING_DIRECTORY+CACHE_DIRECTORY,(new Integer(i)).toString()+URL_EXTENSION);
	ff.delete();
	
      }//for
      bc=new BufferCircolare(0,0,CACHE_SIZE);//resetta il buffer di cache
    }//if
    
    }//performed
  }//CCL
  
  /** gestore eventi del menu file*/
  class ML implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      MenuItem target = (MenuItem)e.getSource();//quale voce  stata scelta
      String actionCommand = 
	target.getActionCommand();//quale comando  ad essa associato
      if(actionCommand.equals("Open")) {//apertura file tramite gestore predefinito dal sistema
	FileDialog d = new FileDialog(
				      CGUI3.this,
				      "Apri file");
	d.setFile("*.*");
	d.setDirectory("."); // Current directory
	d.show();
	String yourFile = "*.*";
	if((yourFile = d.getFile()) != null) {
	  
	  
	  try {//lettura dal file scelto
	    
	    File cf=new File(d.getDirectory(),yourFile);
	    FileInputStream fis = new FileInputStream(cf);
	    byte[] tmpbuf=new byte[(new Long(cf.length())).intValue()];
	    
	    int lg=fis.read(tmpbuf);
			if (verbose) System.out.println("\n read "+lg+" bytes: "+tmpbuf+"\n");
			if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
			String temp=(new String(tmpbuf,0,lg));
			
			//if (verbose) System.out.println("temp= <"+temp+">");  
			
			actiontext.setText(temp);
			fis.close();
	  } catch (Exception er){actiontext.setText(er.toString());}
	  
	  
	  
	  
	  
	  
		    
	  
	} else {
	  actiontext.setText("Operazione annullata");
		    
	}
      } //else
      else if (actionCommand.equals("Save")) {//salvataggio su file, sempre tramite gestore predefinito dal sistema
	FileDialog d = new FileDialog(
				      CGUI3.this,
				      "Salva file");
	d.setFile("*.*");
	d.setDirectory("."); // Current directory
	d.show();
	String yourFile = "*.*";
	if((yourFile = d.getFile()) != null) {
      
	  saveBytesOnFile((actiontext.getText()).getBytes(),d.getDirectory(),yourFile,false);
	} else {
	  actiontext.setText("Operazione annullata");
	  
	}
      }//else
      
      else if(actionCommand.equals("Exit")) {
	
	close_client();//chiusura completa del client
      }
      else if(actionCommand.equals("Clear")) {
	
	actiontext.setText("");//pulisci schermo
	urltext.setText("");
      }
	}
  }
  
  
  
  
  
  /*** Esegue tutte le operazioni di chiusura del client, tra cui la
       distruzione delle finestre e il salvataggio su disco dei dati di configurazione
  ***/
  
  public int close_client()
    {
      try {  //salva dati
	File f= (new File(WORKING_DIRECTORY,CFG_FILE));
	if (f.exists()) {  if (verbose) System.out.println("\n Already present.Overwrite!"); f.delete();}
	//    if (verbose) System.out.println("new_ab: file exists "+f.exists()+" name "+f.getName());  //messaggio 
	FileOutputStream fos=new FileOutputStream(f);
	buf=(fill(bc.getFirst()+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(bc.getLast()+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(ROWS+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(COLS+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(X+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(Y+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(TCP_RECEIVE_BUFFER+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(UDP_RECEIVE_BUFFER+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	
	buf=(fill(CACHE_DIRECTORY+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(CFG_FILE+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(TEMPFILE+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	buf=(fill(SPECIAL_SYMBOL+"\r\n")).getBytes();
	fos.write(buf,0,BYTES_PER_ROW);
	
	
	fos.close();
	if (verbose) System.out.println("new_ab: file exists "+f.exists()+" name "+f.getName());  //messaggio 
	removeAll();	    
	dispose();
	System.exit(0);
	return 0; 
      }catch(Exception ek){if (verbose) System.out.println("ec "+ek);return 1;}
      
      
    }
  
  
  /*** funzione ausiliaria che restituisce in output una stringa di 
       lunghezza costante, costruita a partire da quella in ingresso aggiungendo
       spazi alla fine*/
  String fill(String s)
    {
      int temp=s.length();
      if (temp>BYTES_PER_ROW) return s.substring(0,BYTES_PER_ROW);//sdfsdf
      if (temp==BYTES_PER_ROW) return s;
      temp=BYTES_PER_ROW-temp; //>0
      String res=s;
      for (int i=0;i<temp;i++) res+=" ";
      if (verbose) System.out.println("fill: res='"+res+"' lg="+res.length());
      return  res;
    }
  
  /*** salva sul file specificato i dati in "tosave" : in caso di errore
       torna -1. Salva utilizzando la codifica carattere standard della macchina
    append specifica se appendere i dati o creare un nuovo file
  **/

  int saveOnFile(String tosave,String dir,String name,boolean append)
    {

	//if (verbose) System.out.println("tosave= <<<"+tosave+">>>");
	try {  
	    byte[] buf=new byte[tosave.length()];
	    buf=(tosave).getBytes();
  
	    File file= (new File(dir+File.separator,name));
	    if (verbose) System.out.println("creacc: file exists "+file.exists()+" name "+file.getName());  //messaggio 
	    FileOutputStream fos=new FileOutputStream(dir+File.separator+name,append);
	     if (verbose) System.out.println("creacc: file exists "+file.exists()+" name "+file.getName()+" dir+name="+dir+File.separator+name+" end");  //messaggio   
	    fos.write(buf);
	    fos.close();  
	    
	    
	}catch(Exception eee){if (verbose) System.out.println("saveOnFile :errore sul file "+eee); return -1;}
	
	return 0;
    }

  /*** salva sul file specificato i dati (byte) in "tosave" : in caso di errore
       torna -1
  append  specifica se appendere i dati o creare un nuovo file
  **/
  int saveBytesOnFile(byte tosave[],String dir,String name,boolean append)
    {
      if (verbose) System.out.println("tosave= <<<"+tosave+">>>");
      try {  
	
	    File file= (new File(dir+File.separator,name));
	    if (verbose) System.out.println("creacc: file exists "+file.exists()+" name "+file.getName());  //messaggio 
	    FileOutputStream fos=new FileOutputStream(dir+File.separator+name,append);
	    
	    fos.write(tosave);
	    fos.close();	    
	
      }catch(Exception eee){if (verbose) System.out.println("saveBytes: errore sul file"); return -1;}
      
      return 0;
    }
  
  /** controlla e corregge l'url immessa dall'utente modificando la
      variabile di stato u del client. Torna il numero della porta
      per definire una "well known port" */
  int controlla_url() throws java.net.MalformedURLException
    {
	String toURL=urltext.getText();
	String temp1=urltext.getText();
	if (temp1.startsWith("http-tcp")&&udp.getState()) 
	  {temp1=temp1.substring(8);
	    temp1="http-udp"+temp1;
	    urltext.setText(temp1);
	  }
	if (temp1.startsWith("http-udp")&&tcp.getState()) 
	  {temp1=temp1.substring(8);
	  temp1="http-tcp"+temp1;
	  urltext.setText(temp1);
	  }
	
	
	int i=0;//conterr l'indice del primo carattere della sottostringa col nome del protocollo; udp o tcp
	if (toURL.startsWith("http")||toURL.startsWith("HTTP")) {i=7;}
	if (toURL.startsWith("http-")||toURL.startsWith("HTTP-")) {i=11;}
	toURL=toURL.substring(i);
	if (verbose) System.out.println("analyze: "+toURL); 
	
	u=new URL("http://"+(toURL));
	i=u.getPort();//torna -1 se non riesce a estrarre la porta
	if (i==-1) i=HTTP_STANDARD_PORT;
	if (verbose) System.out.println("host="+u.getHost()+" port="+i+" prot="+u.getProtocol()+" anch="+u.getRef()+" file="+u.getFile());
	return i;
    }
  
  /** legge dal file dei dati che codifica nella stringa ritornata usando
      la codifica standard della macchina su cui il programma gira**/
  String readFromFile(String dir,String filename)
    {
      
      try {
	
	    File cf=new File(dir,filename);
	    FileInputStream fis = new FileInputStream(cf);
	    byte[] tmpbuf=new byte[(new Long(cf.length())).intValue()];//***********************************************************************************
	    
	    int lg=fis.read(tmpbuf);
	    if (verbose) System.out.println("\n read "+lg+" bytes: "+tmpbuf+"\n");
	    if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
	    String temp=(new String(tmpbuf,0,lg));
	    
	    if (verbose) System.out.println("temp= <"+temp+">");  
	    fis.close();
	    return temp;
      } catch (Exception er){if (verbose) System.out.println(er); return "";}
    }
  
  /** restituisce i byte letti dal file**/
  byte [] readBytesFromFile(String dir,String filename)
    {
      
      try {
	
	    File cf=new File(dir,filename);
	    FileInputStream fis = new FileInputStream(cf);
	    byte[] tmpbuf=new byte[(new Long(cf.length())).intValue()];//***********************************************************************************
	    
	    int lg=fis.read(tmpbuf);
	    if (verbose) System.out.println("\n read "+lg+" bytes: "+tmpbuf+"\n");
	    if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
	    String temp=(new String(tmpbuf,0,lg));
	    
	    if (verbose) System.out.println("temp= <"+temp+">");  
	    fis.close();
	    return tmpbuf;
      } catch (Exception er){if (verbose) System.out.println(er); return new byte[0];}
    }
  
  /*************COSTRUTTORE dell'interfaccia grafica
		aggiunge tutti i componenti grafici all'interfaccia, inizializza le variabili
		legge dal file di configurazione i dati utili.
		Accetta da riga di comando 
  */
  
  CGUI3(String args[]) {
    if ((args.length==1)&&(args[0].startsWith("-v"))) {System.out.println("Wow! Si chiacchiera!!");verbose=true;}
    if (verbose) {bc.verbose=true;hh.verbose=true;}
    timeout=DEFAULT_TIMEOUT;
    http_handler hh=new http_handler();
    if (verbose) System.out.println("menu cfg");
    //******menu
    for(int i = 0; i < file.length; i++)
      f.add(file[i]);
    for(int i = 0; i < options.length; i++)
      o.add(options[i]);
    for(int i = 0; i < time.length; i++)
      TimeOut.add(time[i]);
    for(int i = 0; i < helpitems.length; i++)
      help.add(helpitems[i]);
    
    mb1.add(f);
    mb1.add(o);
    mb1.add(help);
    setMenuBar(mb1);
    // fine menu
    
    
    
    
    
    
    /****lettura dei parametri di configurazione
   ******/
		   
		   try
		     {  
		       
		       
		       File cf=new File(WORKING_DIRECTORY+"cache");
		       if (!cf.exists()) {cf.mkdirs();} 
		       
		       
		       cf=new File(WORKING_DIRECTORY,CFG_FILE);
		       FileInputStream fis = new FileInputStream(cf);
		       int lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       String temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
			 
		       Integer l=new Integer(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       int cachefirst=l.intValue();
		       
		       
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       
		       temp=(new String(buf,0,lg));
		       //    fis.close();
		       l=new Integer(temp.trim());
		       if (verbose) System.out.println("\n cache_last = "+l+"\n");
		       int cachelast=l.intValue();   
		       bc=new BufferCircolare(cachefirst,cachelast,CACHE_SIZE);
		       bc.stampa();
		       //*************FINE PARAM CACHE
		       //*** PARAMETRI AVANZATI
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       l=new Integer(temp.trim());
		       if (verbose) System.out.println("\nl= = "+l+"\n");
		       ROWS=l.intValue();
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       l=new Integer(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       COLS=l.intValue();
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       l=new Integer(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       X=l.intValue();
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       l=new Integer(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       Y=l.intValue();
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       l=new Integer(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       TCP_RECEIVE_BUFFER=l.intValue();
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       l=new Integer(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       UDP_RECEIVE_BUFFER=l.intValue();
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       String ls=new String(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       CACHE_DIRECTORY=ls;
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       ls=new String(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       CFG_FILE=ls;
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       ls=new String(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       TEMPFILE=ls;
		       lg=fis.read(buf);
		       if (verbose) System.out.println("\n read "+lg+" bytes: "+buf+"\n");
		       if (verbose) System.out.println("constr: file exists "+cf.exists()+" name "+cf.getName());  
		       temp=(new String(buf,0,lg));
		       
		       if (verbose) System.out.println("temp= <"+temp+">");  
		       
		       ls=new String(temp.trim());
		       if (verbose) System.out.println("\n cache_first = "+l+"\n");
		       SPECIAL_SYMBOL=ls;
		       fis.close();
		       
		     }
		   catch(Exception ec)
		     {
		       if (verbose) System.out.println(ec); 
		       bc=new BufferCircolare(0,0,CACHE_SIZE);
		       bc.stampa();
		       
		     }
		   
		   
		   
		   
		   
		   
		   //aggiunta dei principali componenti
		   
		   setLayout(new FlowLayout());
		   
		   backbut.addActionListener(new backbutL());
		   fwdbut.addActionListener(new fwdbutL());
		   reloadbut.addActionListener(new sendbutL());
		   closebut.addActionListener(new closebutL());
		   sendbut.addActionListener(new sendbutL());
		   listbut.addActionListener(new listbutL());
		   add(backbut);
		   add(fwdbut);
		   add(reloadbut);
		   
		   
		   add(listbut);
		   add(udp);
		   add(tcp);
		   add(urllab);
		   add(urltext);
		   
		   add(actiontext);
		   add(sendbut);
		   add(closebut);  
		   
		   adjust();
		   setForeground(Color.blue);
		   
		   if (verbose) System.out.println("componenti aggiunti");
		   
		   
  }
  
  /** funzione ausiliaria che si occupa di eseguire un programma
      residente sulla macchina locale. La stringa s pu contenere una stringa
      SPECIAL_SYMBOL, definita nelle costanti, a cui sostituisce come
      parametro TEMPFILE, che contiene il file temporaneo con i dati ottenuti
      dal server**/
  
  public void execute(String s)
    {
      StringTokenizer st = new StringTokenizer(s);
      int ct=st.countTokens()+1;
      String[] str=new String[ct];
      ct=0;
      boolean put_file_name_at_the_end=true;
      while (st.hasMoreTokens()) {
	String temp=st.nextToken();
	if (temp.startsWith(SPECIAL_SYMBOL))
	  {
	    temp=TEMPFILE;
	    put_file_name_at_the_end=false;
	    
	  }                
	if (verbose) System.out.println(temp);
	str[ct]=temp;
	    ct++;
      }
      
      if (put_file_name_at_the_end) str[ct]=TEMPFILE;
      Process run;
      try{
	if (verbose) System.out.println("exec: "+str);
	run=Runtime.getRuntime().exec(str);
	run.waitFor();}catch (Exception exc) {
	  if (verbose) System.out.println("exec error");
	  actiontext.setText("File non trovato o risorse di sistema non sufficienti. I dati sono nel file "+TEMPFILE);
	}
      
    }
  
  
  
  
  


  //*** verifica se una url  in cache. Torna "" se non  cos **/
  
  String incache(String url)
    {

	for (int i=0;i<CACHE_SIZE;i++)
	    {
		String url2=readFromFile(WORKING_DIRECTORY+CACHE_DIRECTORY,(new Integer(i)).toString()+URL_EXTENSION);
		if (url.compareTo(url2)==0) return readFromFile(WORKING_DIRECTORY+CACHE_DIRECTORY,(new Integer(i)).toString()/*+URL_EXTENSION*/);
		
		
		
	    }
	return "";
    }
    
  
  
  /***regola l'interfaccia**/
  public void adjust()
    {
      
      setSize(X,Y);
      setResizable(false);
      setBackground(Color.lightGray);
      
      setVisible(true);
      
      backbut.setLocation(LEFT_BORDER,UP_BORDER);
      backbut.setSize(BUT_SIZE,BUT_SIZE);
      fwdbut.setLocation(LEFT_BORDER+BUT_SIZE+GAP,UP_BORDER);
      fwdbut.setSize(BUT_SIZE,BUT_SIZE);
      reloadbut.setLocation(2*BUT_SIZE+2*GAP+LEFT_BORDER,UP_BORDER);
      reloadbut.setSize(BUT_SIZE,BUT_SIZE);
      listbut.setLocation(LEFT_BORDER+3*BUT_SIZE+3*GAP,UP_BORDER);
      listbut.setSize(BUT_SIZE,BUT_SIZE);
      
      actiontext.setLocation(LEFT_BORDER,Y/3);
      urltext.setLocation(LEFT_BORDER+3*GAP,Y/3-2*GAP); 
      closebut.setLocation(X/3*2, Y-DOWN_BORDER);
      closebut.setSize(BUT_SIZE,BUT_SIZE);
      sendbut.setSize(BUT_SIZE,BUT_SIZE);
      sendbut.setLocation(X/3,Y- DOWN_BORDER);
      urllab.setLocation(LEFT_BORDER,Y/3-2*GAP);
      tcp.setLocation(2*X/3,UP_BORDER+GAP);
      udp.setLocation(2*X/3+60,UP_BORDER+GAP);
      
      
    }
  
  
  /** stampa la directory della cache **/    
  public class listbutL implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      actiontext.setText("");
      for (int i=0;i<CACHE_SIZE;i++){
	
	actiontext.setText(actiontext.getText()+"\n"+readFromFile(WORKING_DIRECTORY+CACHE_DIRECTORY,(new Integer(i)).toString()+URL_EXTENSION));
	
      }//for
    }//perform
  }//listener
  
  
  
  /** carica il prossimo file nella cache, fermandosi all'ultimo in 
      ordine cronologico **/   
  public class fwdbutL implements ActionListener 
  {
    
    public void actionPerformed(ActionEvent e) 
      {
	bc.stampa();	
	bc.avanti();
	bc.stampa();
	Integer I=new Integer(bc.getCur());
	
	
	
	actiontext.setText(readFromFile(WORKING_DIRECTORY+CACHE_DIRECTORY,I.toString()));    
	urltext.setText(readFromFile(WORKING_DIRECTORY+CACHE_DIRECTORY,I.toString()+URL_EXTENSION));   
      }
    
  }
  
  
  
  
  /** carica il file precedente nella cache, in ordine cronologico**/
  public class backbutL implements ActionListener 
  {
    public void actionPerformed(ActionEvent e) 
      {
	
	bc.stampa();	
	bc.indietro();
	bc.stampa();
	Integer I=new Integer(bc.getCur());
	
	actiontext.setText(readFromFile(WORKING_DIRECTORY+CACHE_DIRECTORY,I.toString()));    
	urltext.setText(readFromFile(WORKING_DIRECTORY+CACHE_DIRECTORY,I.toString()+URL_EXTENSION));   
	
      }
    
    
  }
  
  /**Invia una richiesta al server, stabilendo se usare UDP o TCP come
     supporto a HTTP**/
  
  public class sendbutL implements ActionListener 
  {
    
    
    /** l'utente vuole tentare una connessione su TCP a seguito del
	fallimento di HTTP su UDP per motivi dimensionali 
	incorpora la gestione della lettura dal socket a blocchi, nel caso in
	cui il buffer tcp non sia abbastanza largo da consentire il prelievo
	dell'intero stream proveniente dal socket. Guardare la gestione in
	actionPerformed di sendbutL
    **/
    public class okL implements ActionListener 
    {
      public void actionPerformed(ActionEvent e)
	{
	
tcp.setState(true);
udp.setState(false);
		preleva_dati(e);
   ok.removeActionListener(okl);
	    no.removeActionListener(nol);
	    f.remove(ok);f.remove(no);f.remove(nlab);
f.dispose();
	  
	}      
    }
    /** l'utente non vuole tentare una connessione su TCP a seguito del
	fallimento di HTTP su UDP per motivi dimensionali **/
    
    public class noL implements ActionListener 
	{
	  public void actionPerformed(ActionEvent e) {
	    ok.removeActionListener(okl);
	    no.removeActionListener(nol);
	    f.remove(ok);f.remove(no);f.remove(nlab);
	    f.dispose();				}
	  
    }
    
    
    
    /** l'utente vuole salvare su disco, nel file specificato nel campo
	testo, i dati (non testuali) ottenuti dalla connessione**/
    
    public class salvaL implements ActionListener 
    {
      public void actionPerformed(ActionEvent e) {
	
	try { 
	    
File newfile=new File(testo.getText());
File oldfile=new File(TEMPFILE);
oldfile.renameTo(newfile);
	  
	  
	  
	}catch(Exception eee){actiontext.setText("errore sul file");}
	
	
	apri.removeActionListener(april);
	salva.removeActionListener(salval);
	annulla.removeActionListener(annullal);
	f.remove(notext);
	f.remove(testo);    
	f.remove(salva);
	f.remove(apri);
	f.remove(annulla);         
	f.dispose();				}
      
    }
    
    /** l'utente vuole utilizzare l'applicazione specificata (con parametri
	eventuali) per accedere ai dati ottenuti**/

    public class apriL implements ActionListener 
    {
      public void actionPerformed(ActionEvent e) {
	
	  //try { 
	    //saveBytesOnFile(hh.bytes,".",TEMPFILE,-1);
	  
	  
	  
	  //}catch(Exception eee){actiontext.setText("errore sul file");}
	
	execute(testo.getText());
		
	apri.removeActionListener(april);
	salva.removeActionListener(salval);
	annulla.removeActionListener(annullal);
	f.remove(notext);
	f.remove(testo);    
	f.remove(salva);
	f.remove(apri);
	f.remove(annulla);         
	f.dispose();				}
      
    }
    /** annulla l'operazione in corso**/
    public class annullaL implements ActionListener 
    {
      public void actionPerformed(ActionEvent e) {
	apri.removeActionListener(april);
	salva.removeActionListener(salval);
	annulla.removeActionListener(annullal);
	f.remove(notext);
	f.remove(testo);    
	f.remove(salva);
	f.remove(apri);
	f.remove(annulla);         	
	f.dispose();				}
      
    }
    
    
    
    /**variabili locali*/	
    Frame f=new Frame("Attenzione!");
    
    Button ok=new Button("S");
    Button no=new Button("No");
    
    Button apri=new Button("Apri con");
    Button salva=new Button("Salva su disco");
    Button annulla=new Button("Annulla");
    Label notext=new Label("File non testuale. Che ci faccio?");
    TextField testo=new TextField(15);
    Label nlab=new Label("UDP non ce la fa. Desideri riprovare con TCP?");
    okL okl=new okL();
    noL nol=new noL();
    apriL april=new apriL();
    salvaL salval=new salvaL();
    annullaL annullal=new annullaL();
    
    
    /** stabilisce se usare TCP o UDP E LO USA, in ingresso prende la porta da usare
	per la connessione. Modifica la variabile hh, torna true se
	un client ha preso un timeout*/
    
    boolean scegli_protocollo(int i) throws java.io.IOException 
      {
	  boolean res=false;
	  if (tcp.getState()) 
	      {
		  TcpClient2 tc2=new TcpClient2();
		  if (verbose) tc2.verbose=true;
		  
		  hh=tc2.Send(u.getHost(),u.getFile(),i,"get",timeout,TCP_RECEIVE_BUFFER);
		
		  //all'interno fa una read
  if (tc2.gotTimeOut){
		      hh=new http_handler(); hh.msg="TIMEOUT"; hh.inbytes=false;
		      res=true;
		  }
		  
		  else {
		      //if (!hh.inbytes) temp=hh.msg;
		      //else temp=new String(hh.bytes);
		  }//timeout				
	      }//if tcp
	  else 
	      {
		
		  UdpClient udpc =new UdpClient(1,u.getHost(),u.getFile(),i,"GET",timeout,UDP_RECEIVE_BUFFER);
		  if (verbose) udpc.verbose=true;		    
		  udpc.run();       
		  if (udpc.gotTimeOut)
		      {if (verbose) System.out.println("UDP TIMEOUT");
		      hh=new http_handler();
		      res=true;
		      hh.msg="TimeOut!"; hh.inbytes=false;}
		  else {
		      ByteArrayInputStream bais=
			  new ByteArrayInputStream(udpc.bytes);
		      hh=new http_handler(bais,udpc.BUFFER_SIZE);
		      hh.more=false;
		      hh.read();
		      hh.more=false;
		      hh.authoritative=true;}
		  if (verbose) System.out.println("udpc invocato...");
		  
	      }//else tcp
	  return res;    
      }//fine scegli protocollo
    
    
    /** controlla la eventuale presenza di un errore 444 "pacchetto troppo
	grande per UDP", allo scopo di chiedere all'utente se vuole ritentare
	una connessione TCP**/
    
    boolean controlla_444()
      {
	if (udp.getState()&&((hh.return_code).startsWith("444"))) 
	  {
	    
	    f.setLayout(new FlowLayout());
	    
	    ok.addActionListener(okl);
	    no.addActionListener(nol);
	    f.add(nlab);
	    
	    f.add(ok);
	    f.add(no);
	    f.setSize(450,100);
	    f.setLocation(320,160);
	    ok.setLocation(360,70);
	    no.setLocation(300,70);
	    ok.setSize(40,40);
	    no.setSize(40,40);
	    f.setBackground(Color.red);
	    f.setVisible(true);
	    f.requestFocus();
	    f.addWindowListener(
				new WindowAdapter() {
	      public void windowClosing(WindowEvent e){
		if (verbose) System.out.println("Window Closing");
		ok.removeActionListener(okl);
		no.removeActionListener(nol);
		f.remove(ok);f.remove(no);f.remove(nlab);
		f.dispose();
	      }
	    });
	    
	    return true;    
	  }// if 444
	return false;
	}//fine ctrl444
    
    
    /** Visualizza le risorse JPEG. Utilizza delle classi presenti solo
	in Java 1.2: se utilizzato con versioni precedenti genera una
	eccezione NoClassDefFoundError, gestita a livello pi alto**/
    void show_jpeg(String nomefile)
      { 
	try { 
	  
	    
	    //imager I=new imager(400,400,urltext.getText());
	    //if (verbose) imager.verbose=true;
	    //puppet p=new puppet(nomefile,200,200);
	    //I.addPuppet(0,p);
	    
	  
	  //f.add(I);		    
	}
	catch(NoClassDefFoundError ncdf) 
	  {actiontext.setText("versione di Java non adatta alla visualizzazione");if (verbose) System.out.println(ncdf);}  
	catch(Exception eee){if (verbose) System.out.println(eee);
	actiontext.setText("errore sul file di swap");}
      }	
    
    /** salva la risorsa in cache

se appendo NON avanzo
*/
    void scrivi_cache(String dati,boolean append)
      {
	Integer I=new Integer(bc.getLast());
	
	bc.stampa();
	if (!append) bc.scrivi();
	bc.stampa();
	
	//if (verbose) System.out.println("recvd msg=<<<"+dati+">>>");
	saveOnFile(dati,WORKING_DIRECTORY+CACHE_DIRECTORY, I.toString(),append);

	saveOnFile(urltext.getText(),WORKING_DIRECTORY+CACHE_DIRECTORY, I.toString()+URL_EXTENSION,false);
	
	
      }//fine scrivi cache
    
    
    /** abbiamo ricevuto dati non testuali.
	mostriamo le scelte possibili all'utente*/
    
    void chiedi_azione()
      {
	Dialog d=new Dialog(f,"Ed ora che si fa?",true);
	if (verbose) System.out.println("OOO");
	d.setLayout(new FlowLayout());
	
	salva.addActionListener(salval);
	apri.addActionListener(april);
	annulla.addActionListener(annullal);
	
	d.add(notext);
	d.add(testo);    
	d.add(salva);
	d.add(apri);
	d.add(annulla);
	
	d.setSize(450,100);
	d.setLocation(320,160);
	d.setBackground(Color.lightGray);
	d.setVisible(true);
	d.requestFocus();
	d.addWindowListener(
			    new WindowAdapter() {
	  public void windowClosing(WindowEvent e){
	    if (verbose) System.out.println("Window Closing");
	    apri.removeActionListener(april);
	    salva.removeActionListener(salval);
	    annulla.removeActionListener(annullal);
	    //    d.remove(notext);
	    //   d.remove(testo);    
	    //   d.remove(salva);
	    //   d.remove(apri);
	    //   d.remove(annulla);         
	    //    d.dispose();
	    dispose();
	  }
	});
	
      }//fine chiedi azione
    

      /***
	  Questa funzione si occupa della comunicazione col server, scegliendo il protocollo adatto e 
	  gestendo un eventuale prelievo a blocchi dei dati letti da http_handler
      **/
    
    void preleva_dati(ActionEvent e)
	{

	    try{
	    int i=controlla_url();
	    
	    hh=new http_handler();
	    hh.msg="";
	    hh.msg=incache(urltext.getText());
	    
	    if (verbose) System.out.println("CGUI 965");

	    /* cosa  stato spinto?*/
	    Button target = (Button)e.getSource();
	    //se era un reload o non era in cache
	    boolean was_a_timeout=false;
	    boolean was_a_444=false;
	    if (((target.getLabel())!="Reload")&&hh.msg!="") 
	      {
		
		if (verbose) System.out.println("non e' reload, hh.msg non e' vuoto");   
		hh.inbytes=false;
		hh.more=false;
		hh.size=(hh.msg).length();
		hh.return_code="200";
		hh.authoritative=false;
		if (verbose) System.out.println(" search in cache... msg="+hh.msg);
	      }		
	    else
	      //era reload oppure non stava in cache
	      {
		if (verbose) System.out.println("reload oppure non in cache msg=<"+hh.msg+">");
		/*** attenzione! scegli_protocollo 
		     si occupa anche della comunicazione vera e propria, settando
		     opportunamente la variabile di stato hh**/ 
		was_a_timeout= scegli_protocollo(i);
		if (verbose) System.out.println("controllo 444");
		was_a_444=controlla_444();
		
		
		if (verbose) System.out.println("\ndopo 444");
		
		
	      }//**********
	    int block_count=0;//conta i blocchi di dati
	    //se non ci sono errori  stata fatta una read sullo stream	    
	    //se il blocco  uno solo
	    if (!hh.more)
	      {if (verbose) System.out.println("GUI3 NO MORE!");
	      
	      if (!hh.inbytes) 
		{if (verbose) System.out.println("testo puro!");
		actiontext.setText(hh.msg);
		saveOnFile(hh.msg,".",TEMPFILE,false);
		{   if (!was_a_timeout&&!was_a_444) scrivi_cache(hh.msg,false);}
		}//if !bytes		    
	      else //inbytes
		{saveBytesOnFile(hh.bytes,".",TEMPFILE,false);
		scrivi_cache("risorsa non testuale",false);
		  if (hh.type.equals("image/jpeg")||hh.type.equals("image/jpg")) 
		    {show_jpeg(TEMPFILE);}
		  chiedi_azione();
		}
	      
	      }//if !more
	    else//no more, occorre gestire i dati a blocchi
	      {
		  if (verbose) System.out.println("GUI3 MORE! block "+block_count);
	      
		  if (!hh.inbytes) 
		      {if (verbose) System.out.println("testo puro!");
		      actiontext.setText(hh.msg);
		      
		      saveOnFile(hh.msg,".",TEMPFILE,false);
		        if (!was_a_444) scrivi_cache(hh.msg,true);
		      }
		  else 
		      {if (verbose) System.out.println("Dati in bytes!");
		      
		      scrivi_cache("risorsa non testuale",true);
		      saveBytesOnFile(hh.bytes,".",TEMPFILE,false);
			  
		      }
	     
		  //dopo aver preso il primo blocco passo ai successivi
		  
		StatusBar bar=new StatusBar(1,1,300,60,60,(double)(((double)hh.len)/((double)TCP_RECEIVE_BUFFER)));
		bar.CHAR='|';
		  while (hh.more)
		      {
			
			  hh.reRead();
			  if (verbose) System.out.println("Altro Blocco ct="+block_count);
			  block_count++;
			  if (hh.inbytes)	
			      {  
bar.add();
			      //actiontext.setText(actiontext.getText()+" blocco "+block_count);
				  saveBytesOnFile(hh.bytes,".",TEMPFILE,true);
			      }
			  else
			      { 
				  
				        	   actiontext.setText(actiontext.getText()+hh.msg);
				  saveOnFile(hh.msg,".",TEMPFILE,true);
				  if (!was_a_444) scrivi_cache(hh.msg,true);
			      }	
			  
			  
		      } //while more
		  bar.dispose();
  bc.scrivi();//l'indice ora deve avanzare
			

		  if (hh.type.equals("image/jpeg")||hh.type.equals("image/jpg")) 
		      {show_jpeg(TEMPFILE);
		      chiedi_azione();
		      } 
		  else {   if(hh.inbytes) chiedi_azione();}		
		
		if (block_count>1) {if (verbose) System.out.println("GUI :shutdown");hh.shutdown();}
		//*****************}//recv="" se non sta in cache
	      }//else di gestione blocchi		
	    
	    /*se il risultato non  stato ottenuto da
	      una connessione reale*/
		
		if (!hh.authoritative&&!was_a_timeout&&(!(actiontext.getText()).endsWith("ERROR")))  {
		  if (!(actiontext.getText()).endsWith("FROM CACHE")) actiontext.setText(actiontext.getText()+"\n FROM CACHE");
		  
		}//if !hh
		
	    }//try
	  /* gestione delle eccezioni:*/
	  catch(UnknownHostException eee) 
	      { if (verbose) System.out.println(eee);
	      actiontext.setText("Host sconosciuto...");
	      }//fine catch
	  
	  catch(InterruptedIOException eee) 
	      { if (verbose) System.out.println(eee);
	      actiontext.setText("TimeOut!!");
	      }//fine catch
	  catch(ConnectException eee) 
	  { if (verbose) System.out.println(eee);
	  actiontext.setText("Il server non accetta connessioni. Verificare la porta.");
	  }//fine catch	catch(ConnectException eee) 
	  catch(NoClassDefFoundError eee)	    
	      { if (verbose) System.out.println(eee);
	      actiontext.setText("La versione di Java in uso non definisce alcune classi necessarie.");
	      }//fine catch
	  
	  catch(Exception exc) 
	      { if (verbose) System.out.println(exc);
	      actiontext.setText("errore nella risorsa");
	      }//fine catch
	  
	    }//fine preleva_dati
    
    
    /** Gestione del bottone SEND**/
    
    public void actionPerformed(ActionEvent e) 
      {
	
	
	  
	preleva_dati(e);
      
}//actionperformed
    }//fine sendbutlistener
  
      
      
      
  
      public class closebutL implements ActionListener {
	  public void actionPerformed(ActionEvent e) {
	      close_client();
	      dispose();
	      System.exit(0);
	  }
      }
      
  
      public static void main(String[] args) {
	  
	  CGUI3 a = new CGUI3(args);
    
	  a.setTitle("                                          GFG 1.0");
    
    
    a.addWindowListener(
			new WindowAdapter() {
      public void windowClosing(WindowEvent e){
	if (verbose) System.out.println("Window Closing");
	
	
      }
    }); 
    
    a.adjust();	
    
    
  }
}

   








