

import java.lang.Thread;
import java.net.*;
import java.io.*;

/********************
		     Client UDP per il server GFG1.0
		     Definisce un thread che invia una richiesta al server, costruendola
		     con la funzione pacchetto_richiesta, e attende, per un tempo massimo
		     specificato in timeout, la risposta del server. I dati ricevuti sono
		     poi letti realmente da http_handler, al quale viene passato il valore
		     del buffer massimo utilizzabile in lettura.
		     Per UDP valori del buffer superiori alla massima dimensione del
		     pacchetto che pu transitare sulla rete non hanno molto senso.
		     Viene utilizzata la classe Dgram che costruisce datagrammi a partire
		     da dati.
***********************/
public class UdpClient extends Thread {
  
  public static final String UDP_STRING="HTTP-UDP/1.0";
  public static final String TCP_STRING="HTTP-TCP/1.0";
  public  int BUFFER_SIZE=1500;
  public static boolean verbose=false;
  
  private DatagramSocket s;
  private InetAddress hostAddress;
  public byte[] bytes;
  private int timeout;
  public boolean gotTimeOut;
  private byte buf[]=new byte[BUFFER_SIZE];
  private DatagramPacket dp = 
    new DatagramPacket(buf, BUFFER_SIZE);
  private int id;
  
  public int port;
  public String resource;
  public String method;
  public UdpClient()
    { gotTimeOut=false; timeout=3000;}
  public UdpClient(int identifier,String hh,String r,int po,String m,int tim,int bufsize) {
    id = identifier;timeout=tim;
    port=po; resource=r; method=m;
    if (bufsize>0) BUFFER_SIZE=bufsize;
    try {
      s = new DatagramSocket();//porta assegnata dal sistema
      hostAddress = 
        InetAddress.getByName(hh);
    } catch(UnknownHostException e) {
      System.err.println("host sconosciuto");
      //System.exit(1);
    } catch(SocketException e) {
      System.err.println("Can't open socket");
      e.printStackTrace();
      //System.exit(1);
    } 
    if (verbose) System.out.println("Client starts");
  }

  public void run() {    
    try {
      for(int i = 0; i < 1; i++) {
	
	InetAddress ia=s.getLocalAddress();
        buf=(http_handler.pacchetto_richiesta(resource,"UDP",method,ia.getHostAddress())).getBytes();

	if (verbose) System.out.println("RICH:\n"+(new String(buf,0,buf.length)));
        s.send(Dgram.toDatagram(buf,
				hostAddress, 
				port));
        // Bloccante!!
	if (verbose) System.out.println("udpc sent data");
	bytes=new byte[BUFFER_SIZE];
	dp=new DatagramPacket(bytes,BUFFER_SIZE);
	
	s.setSoTimeout(timeout);      
	s.receive(dp);
	
	//bytes=dp.getData();
	if (verbose) System.out.println("udpc recvd data");
	String message=Dgram.toString(dp);        
	String rcvd = "Client #" + id +
	  ", recvd from " + 
	  dp.getAddress() + ", " + 
          dp.getPort() + " <" +message+">";
	
	
        if (verbose) System.out.println(rcvd);
      }
    } 
    catch(InterruptedIOException se) {if (verbose) System.out.println("UdpCL: TIMEOUT");
    gotTimeOut=true;}
    catch(IOException e) {
      e.printStackTrace();if (verbose) System.out.println(" in UdpClient ");
      //System.exit(1);
    }
  }
  public static void main(String[] args) {
    //for(int i = 0; i < 10; i++)
    // new UdpClient().start();
  }
} 




